/*

GooBer CPU core. 
Copyright (C) 2002 Ian Smith

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __GBCPU_H__
#define __GBCPU_H__

#include <stdio.h>
#include "defs.h"

#define CPU_STOP   2
#define CPU_HALT   1
#define CPU_NORMAL 0

/* Typedef function pointers, GBREAD reads a byte from memory,
   GBWRITE writes a byte to memory */
typedef unsigned char (*GBREAD)(unsigned short);
typedef void (*GBWRITE)(unsigned short, unsigned char);

/* Structure to hold the state of the CPU */
typedef struct _gbcpu_context
{
  unsigned short pc, sp;
  unsigned char flags, a, b, c, d, e, h, l;
  unsigned short cycles;
  int cpu_mode;
  int ime;
  GBREAD  read;
  GBWRITE write;
} gbcpu_context;

/* Functions */

/* Reset the GameBoy CPU */
void gbcpu_reset(gbcpu_context *gbcpu);

/* Execute 1 instruction */
int gbcpu_ex(gbcpu_context *gbcpu);

/* Set an interrupt */
int gbcpu_int(gbcpu_context *gbcpu, unsigned short addr);

/* Disassemble current instruction */
void gbcpu_da(gbcpu_context *gbcpu, char out[]);

#endif









