/*

GooBer Portable gameboy emulator
Copyright (C) 2002 Ian Smith

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "io.h"
#include "mem.h"
#include "lcdc.h"
#include "intr.h"
#include "joypad.h"
#include "timer.h"

/* Array of bytes to catch any unimplemented I/O reg values */
unsigned char io[0x7F] = {0};

void io_init()
{
  io_write(0x5,  0x00);
  io_write(0x6,  0x00);
  io_write(0x7,  0x00);
  io_write(0x10, 0x80);
  io_write(0x11, 0xBF);
  io_write(0x12, 0xF3);
  io_write(0x14, 0xBF);
  io_write(0x16, 0x3F);
  io_write(0x17, 0x00);
  io_write(0x19, 0xBF);
  io_write(0x1A, 0x7F);
  io_write(0x1B, 0xFF);
  io_write(0x1C, 0x9F);
  io_write(0x1E, 0xBF);
  io_write(0x20, 0xFF);
  io_write(0x21, 0x00);
  io_write(0x22, 0x00);
  io_write(0x23, 0xBF);
  io_write(0x24, 0x77);
  io_write(0x25, 0xF3);
  io_write(0x26, 0xF1);
  io_write(0x40, 0x91);
  io_write(0x41, 0x80);
  io_write(0x42, 0x00);
  io_write(0x43, 0x00);
  io_write(0x45, 0x00);
  io_write(0x47, 0xFC);
  io_write(0x48, 0xFF);
  io_write(0x49, 0xFF);
  io_write(0x4A, 0x00);
  io_write(0x4B, 0x00); 

  set_io_handlers(io_read, io_write);

}

unsigned char io_read(unsigned short addr)
{
  /* Joypad reg */
  if (addr == 0) {
    return joypad_read_reg();
  }
  /* Timer regs */
  else if ( (addr >= 0x04) && (addr <= 0x07) ) {
    return timer_read_reg(addr);
  }
  /* LCDC Regs */
  else if ((addr >= 0x40) && (addr <= 0x4B)) {
    return lcdc_read_reg(addr);
  }
  /* Joypad reg */
  else if (addr == 0) {
    return 0x0F;
  }
  /* IF */
  else if (addr == 0x0F) {
    return intr_read_reg(addr);
  }
  else {
    return io[addr];
  }
}

void io_write(unsigned short addr, unsigned char value)
{
  if (addr == 0x00) {
    joypad_write_reg(value);
    return;
  }

  lcdc_write_reg(addr, value);
  intr_write_reg(addr, value);
  timer_write_reg(addr, value);

  io[addr] = value;
}



