/*

Goober Portable Gameboy Emulator
Copyright (C) 2002 Ian Smith

This Program Is Free Software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

//#include <stdio.h>
//#include <stdlib.h>

#include "joypad.h"
#include "intr.h"

#define DPAD_SELECTED    1
#define BUTTONS_SELECTED 2

/* Returns TRUE if KEY is in the keyboard buffer */
#define PRESSED(KEY) (key[joypad_keys[KEY]])

/* Returns TRUE if BUTTON has changed from high to low */
#define CHANGED(BUTTON) ( (PRESSED(BUTTON)) && (!pressed_keys[BUTTON]) )

/* Holds keys to represent joypad buttons */
static int joypad_keys[8] = 
  { KEY_LEFT, KEY_RIGHT, KEY_UP, KEY_DOWN, 
    KEY_ENTER,     /* START */
    KEY_QUOTE,     /* SELECT */
    KEY_ALT,       /* A */
    KEY_LCONTROL   /* B */
  };

static int pressed_keys[8] = {0};

static int selected_keys=0;

void joypad_init()
{
  const char *layout = NULL;

  selected_keys=0;

  /* get desired Joypad layout */
  layout = get_config_string("controller", "layout", "default_layout");

  /* read joypad keys */
  joypad_keys[INPUT_DOWN]  = get_config_int(layout, "down",  KEY_DOWN);
  joypad_keys[INPUT_UP]    = get_config_int(layout, "up",    KEY_UP);
  joypad_keys[INPUT_LEFT]  = get_config_int(layout, "left",  KEY_LEFT);
  joypad_keys[INPUT_RIGHT] = get_config_int(layout, "right", KEY_RIGHT);

  joypad_keys[START]  = get_config_int(layout, "start",  KEY_ENTER);
  joypad_keys[SELECT] = get_config_int(layout, "select", KEY_QUOTE);

  joypad_keys[BUTTON_B] = get_config_int(layout, "b_button", KEY_LCONTROL);
  joypad_keys[BUTTON_A] = get_config_int(layout, "a_button", KEY_ALT);

}

void joypad_cleanup()
{
}

unsigned char joypad_read_reg()
{
  unsigned char ret=0;

  //  poll_keyboard();

  switch(selected_keys) {
  case DPAD_SELECTED:
    if (PRESSED(INPUT_DOWN))  ret |= P13;
    if (PRESSED(INPUT_UP))    ret |= P12;
    if (PRESSED(INPUT_LEFT))  ret |= P11;
    if (PRESSED(INPUT_RIGHT)) ret |= P10;
    break;

  case BUTTONS_SELECTED:
    if (PRESSED(START))    ret |= P13;
    if (PRESSED(SELECT))   ret |= P12;
    if (PRESSED(BUTTON_B)) ret |= P11;
    if (PRESSED(BUTTON_A)) ret |= P10;
    break;

  default:
    ret = 0x0F;
    break;
  }

  return ~ret;
}

void joypad_write_reg(unsigned char value)
{
  switch(value) {
  case 0x20: /* selected P14 */
    selected_keys = DPAD_SELECTED;
    break;

  case 0x10: /* selected P15 */
    selected_keys = BUTTONS_SELECTED;
    break;
  }
}

void joypad_update()
{
  if (CHANGED(INPUT_DOWN)) intr_request(INTR_JOYPAD);
  if (CHANGED(INPUT_UP)) intr_request(INTR_JOYPAD);
  if (CHANGED(INPUT_LEFT)) intr_request(INTR_JOYPAD);
  if (CHANGED(INPUT_RIGHT)) intr_request(INTR_JOYPAD);

  if (CHANGED(START)) intr_request(INTR_JOYPAD);
  if (CHANGED(SELECT)) intr_request(INTR_JOYPAD);
  if (CHANGED(BUTTON_B)) intr_request(INTR_JOYPAD);
  if (CHANGED(BUTTON_A)) intr_request(INTR_JOYPAD);

  pressed_keys[INPUT_DOWN] = PRESSED(INPUT_DOWN);
  pressed_keys[INPUT_UP] = PRESSED(INPUT_UP);
  pressed_keys[INPUT_LEFT] = PRESSED(INPUT_LEFT);
  pressed_keys[INPUT_RIGHT] = PRESSED(INPUT_RIGHT);

  pressed_keys[START] = PRESSED(START);
  pressed_keys[SELECT] = PRESSED(SELECT);
  pressed_keys[BUTTON_B] = PRESSED(BUTTON_B);
  pressed_keys[BUTTON_A] = PRESSED(BUTTON_A);

}
